"use strict";
/**
 * @module webdnn
 */
/** Don't Remove This comment block */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var localforage = require("localforage");
var buffer_webmetal_1 = require("../buffer/buffer_webmetal");
var get_weight_decoder_1 = require("../decoder/get_weight_decoder");
var fetch_1 = require("../fetch");
var placeholder_1 = require("../placeholder");
var symbolic_float32array_1 = require("../symbolic_typed_array/symbolic_float32array");
var webdnn_1 = require("../webdnn");
var webmetal_handler_1 = require("../webmetal_handler");
var descriptor_runner_1 = require("./descriptor_runner");
/**
 * Check this device is iOS devices or not.
 * @private
 */
var IS_IOS = navigator.userAgent.includes('iPhone') || navigator.userAgent.includes('iPad');
/**
 * DescriptorRunner for WebMetal
 * @protected
 */
var DescriptorRunnerWebMetal = /** @class */ (function (_super) {
    __extends(DescriptorRunnerWebMetal, _super);
    function DescriptorRunnerWebMetal(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, options) || this;
        /**
         * backend name
         */
        _this.backendName = 'webgpu'; // will be changed to 'webmetal' in future version
        return _this;
    }
    /**
     * Return `true` if this backend is available in this environment.
     * @returns {boolean}
     */
    DescriptorRunnerWebMetal.checkAvailability = function () {
        return webmetal_handler_1.IS_WEBMETAL_SUPPORTED;
    };
    /**
     * Initialize descriptor runner asynchronously
     * @returns {Promise<void>} Promise object which is resolved when the initialization finished.
     */
    DescriptorRunnerWebMetal.prototype.init = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.webmetalHandler = webmetal_handler_1.default.getInstance();
                        return [4 /*yield*/, this.checkIncompatibleGPU()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Check whether current GPU is supported or not. If it's not supported, an error is thrown.
     * @returns {Promise<void>}
     */
    DescriptorRunnerWebMetal.prototype.checkIncompatibleGPU = function () {
        return __awaiter(this, void 0, void 0, function () {
            var buffer, threadExecutionWidth;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        /**
                         * It is reported that AMD GPU crashes when performing sgemm (matrix multiplication).
                         * Until this problem is solved, blocking WebMetal backend in the environment is needed.
                         * API in WebMetal does not directly gives hardware information, so trying to determine hardware heuristically.
                         *
                         * Criteria: thread_execution_width == 32 is required
                         * (on AMD FirePro D500, thread_execution_width == 64)
                         *
                         * @see https://github.com/mil-tokyo/webdnn/issues/286
                         */
                        this.webmetalHandler.loadKernel("\n#include <metal_stdlib>\nusing namespace metal;\n        kernel void check_compatibility(\n            device uint *A[[buffer(0)]],\n            uint global_index[[thread_position_in_grid]],\n            uint thread_execution_width[[thread_execution_width]]\n        ){\n            if (global_index == 0) {\n                A[0] = thread_execution_width;\n            }\n        }", 'basic');
                        buffer = this.webmetalHandler.createBuffer(new Uint32Array(1));
                        return [4 /*yield*/, this.webmetalHandler.executeSinglePipelineState('basic.check_compatibility', { width: 1, height: 1, depth: 1 }, { width: 1, height: 1, depth: 1 }, [buffer], true)];
                    case 1:
                        _a.sent();
                        threadExecutionWidth = (new Uint32Array(buffer.contents))[0];
                        if (threadExecutionWidth != 32) {
                            throw new Error("Sorry, this GPU does not compatible with WebMetal (thread_execution_width == " + threadExecutionWidth + ". See checkIncompatibleGPU method of https://github.com/mil-tokyo/webdnn/blob/master/src/descriptor_runner/descriptor_runner/descriptor_runner_webmetal.ts");
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Fetch graph descriptor from specified directory.
     *
     * @param directory directory where descriptor is contained.
     * You can also provide URL of other domain like this.
     *
     * ```javascript
     * await runner.load('://my.other.domain.com/my_model');
     * ```
     *
     * However sometimes it can't because of Cross-Origin-Resource-Security policy.
     *
     * @protected
     */
    DescriptorRunnerWebMetal.prototype.fetchDescriptor = function (directory) {
        return __awaiter(this, void 0, void 0, function () {
            var res;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fetch_1.default(directory + "/graph_" + this.backendName + ".json", this.transformUrlDelegate)];
                    case 1:
                        res = _a.sent();
                        return [2 /*return*/, res.json()];
                }
            });
        });
    };
    /**
     * Fetch parameter files from specified directory.
     *
     * @param directory directory where descriptor is contained.
     * You can also provide URL of other domain like this.
     *
     * ```javascript
     * await runner.load('://my.other.domain.com/my_model');
     * ```
     *
     * However sometimes it can't because of Cross-Origin-Resource-Security policy.
     *
     * @param progressCallback callback which is called to notice the loading is progressing.
     * @protected
     */
    DescriptorRunnerWebMetal.prototype.fetchParameters = function (directory, progressCallback) {
        return __awaiter(this, void 0, void 0, function () {
            var res;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fetch_1.default(directory + "/weight_" + this.backendName + ".bin", this.transformUrlDelegate)];
                    case 1:
                        res = _a.sent();
                        return [2 /*return*/, fetch_1.readArrayBufferProgressively(res, progressCallback)];
                }
            });
        });
    };
    /**
     * Load cached descriptor from WebStorage
     * @protected
     */
    DescriptorRunnerWebMetal.prototype.restoreCachedDescriptor = function (directory) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, localforage.getItem(directory + "_" + this.backendName + "_descriptor").catch(function () { return null; })];
            });
        });
    };
    /**
     * Load cached descriptor from WebStorage
     * @protected
     */
    DescriptorRunnerWebMetal.prototype.restoreCachedParameters = function (directory, progressCallback) {
        return __awaiter(this, void 0, void 0, function () {
            var parameter;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, localforage.getItem(directory + "_" + this.backendName + "_parameters").catch(function () { return null; })];
                    case 1:
                        parameter = _a.sent();
                        if (parameter && progressCallback)
                            progressCallback(parameter.byteLength, parameter.byteLength);
                        return [2 /*return*/, parameter];
                }
            });
        });
    };
    /**
     * save cache
     */
    DescriptorRunnerWebMetal.prototype.saveCache = function (directory, descriptor, parameters) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.all([
                            localforage.setItem(directory + "_" + this.backendName + "_descriptor", descriptor),
                            localforage.setItem(directory + "_" + this.backendName + "_parameters", parameters)
                        ])];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ;
    DescriptorRunnerWebMetal.prototype.setDescriptorAndParameters = function (descriptor, parameter) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.descriptor = descriptor;
                        //reset all datum depend on old descriptor
                        this.staticBuffer = null;
                        this.dynamicBuffer = null;
                        this.metaBuffers = null;
                        this.placeholderContext = new placeholder_1.default(descriptor.placeholders);
                        this.executionInfos = descriptor.exec_infos;
                        //compile kernels
                        this.webmetalHandler.loadKernel(this.descriptor.kernel_source, 'descriptor');
                        return [4 /*yield*/, this.initializeStaticBuffer(parameter)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.initializeMetaBuffers()];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this.setPlaceholderValue({
                                '__MAX_THREADS_PER_THREADGROUP__': IS_IOS ? 512 : 1024
                            })];
                    case 3:
                        _a.sent();
                        if (!(this.placeholderContext && this.placeholderContext.isResolved)) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.initializeDynamicBuffer()];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Initialize static buffers, whose size and position can be determined in compile time.
     *
     * @param {ArrayBuffer} weightRawArray constant weight buffer
     * @returns {Promise<void>}
     */
    DescriptorRunnerWebMetal.prototype.initializeStaticBuffer = function (weightRawArray) {
        return __awaiter(this, void 0, void 0, function () {
            var descriptor, staticBuffer, decoder, _a, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (!this.descriptor)
                            throw Error("GraphDescriptor is not loaded.");
                        descriptor = this.descriptor;
                        staticBuffer = new buffer_webmetal_1.default(descriptor.memory_layout.static.size * Float32Array.BYTES_PER_ELEMENT);
                        this.staticBuffer = staticBuffer;
                        decoder = get_weight_decoder_1.default(descriptor.weight_encoding);
                        _b = (_a = staticBuffer).write;
                        return [4 /*yield*/, decoder.decode(new Uint8Array(weightRawArray))];
                    case 1: return [4 /*yield*/, _b.apply(_a, [_c.sent()])];
                    case 2:
                        _c.sent();
                        return [4 /*yield*/, this.getInputViews()];
                    case 3:
                        (_c.sent())
                            .filter(function (view) { return !view.isDynamic; })
                            .forEach(function (view) {
                            view.buffer = staticBuffer.bufferView.buffer;
                        });
                        return [4 /*yield*/, this.getOutputViews()];
                    case 4:
                        (_c.sent())
                            .filter(function (view) { return !view.isDynamic; })
                            .forEach(function (view) {
                            view.buffer = staticBuffer.bufferView.buffer;
                        });
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Initialize meta buffers, which contains metadata shared in each GPU kernel thread (ex. hyper parameters).
     * @returns {Promise<void>}
     */
    DescriptorRunnerWebMetal.prototype.initializeMetaBuffers = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!this.descriptor)
                            throw Error("GraphDescriptor is not loaded.");
                        _a = this;
                        return [4 /*yield*/, Promise.all(this.descriptor.exec_infos.map(function (executionInfo) { return __awaiter(_this, void 0, void 0, function () {
                                var buffer;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            buffer = new buffer_webmetal_1.default(executionInfo.meta_buffer.length * Int32Array.BYTES_PER_ELEMENT);
                                            return [4 /*yield*/, buffer.write(new Uint8Array(executionInfo.meta_buffer))];
                                        case 1:
                                            _a.sent();
                                            return [2 /*return*/, buffer];
                                    }
                                });
                            }); }))];
                    case 1:
                        _a.metaBuffers = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Initialize dynamic buffers, whose size and position cannot be determined without runtime-information such as input image size
     * (if it's dynamic).
     * When all placeholder is resolved, this method is automatically called.
     *
     * @returns {Promise<void>}
     */
    DescriptorRunnerWebMetal.prototype.initializeDynamicBuffer = function () {
        return __awaiter(this, void 0, void 0, function () {
            var descriptor, placeholderContext, dynamicBufferSize, dynamicBuffer;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.descriptor)
                            throw Error("GraphDescriptor is not loaded.");
                        if (!this.placeholderContext)
                            throw Error("PlaceholderContext is not initialized.");
                        descriptor = this.descriptor;
                        placeholderContext = this.placeholderContext;
                        dynamicBufferSize = placeholderContext.resolve(descriptor.memory_layout.dynamic.size);
                        dynamicBuffer = new buffer_webmetal_1.default(dynamicBufferSize * Float32Array.BYTES_PER_ELEMENT);
                        this.dynamicBuffer = dynamicBuffer;
                        return [4 /*yield*/, this.getInputViews()];
                    case 1:
                        (_a.sent())
                            .filter(function (view) { return view.isDynamic; })
                            .forEach(function (view) {
                            view.buffer = dynamicBuffer.bufferView.buffer;
                        });
                        return [4 /*yield*/, this.getOutputViews()];
                    case 2:
                        (_a.sent())
                            .filter(function (view) { return view.isDynamic; })
                            .forEach(function (view) {
                            view.buffer = dynamicBuffer.bufferView.buffer;
                        });
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Set actual value into placeholder. If all placeholder is resolved,
     * [[DescriptorRunnerWebMetal#initializeDynamicBuffer|`initializeDynamicBuffer()`]] is automatically called.
     *
     * @param values mapping object of placeholder name and value
     * @returns {Promise<void>}
     */
    DescriptorRunnerWebMetal.prototype.setPlaceholderValue = function (values) {
        return __awaiter(this, void 0, void 0, function () {
            var placeholderContext, descriptor, metaBuffers, _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!this.placeholderContext)
                            throw new Error('PlaceholderContext is not initialized.');
                        placeholderContext = this.placeholderContext;
                        placeholderContext.update(values);
                        if (!placeholderContext.isResolved)
                            return [2 /*return*/];
                        if (!this.descriptor)
                            throw new Error('Descriptor is not loaded');
                        if (!this.metaBuffers)
                            throw new Error('MetaBuffers are not initialized');
                        descriptor = this.descriptor;
                        metaBuffers = this.metaBuffers;
                        return [4 /*yield*/, this.initializeDynamicBuffer()];
                    case 1:
                        _b.sent();
                        // resolve placeholders in execution info
                        _a = this;
                        return [4 /*yield*/, Promise.all(descriptor.exec_infos.map(function (executionInfo, i) { return __awaiter(_this, void 0, void 0, function () {
                                var bufferView, _i, _a, unresolved_value;
                                return __generator(this, function (_b) {
                                    bufferView = new Int32Array(metaBuffers[i].bufferView.buffer);
                                    for (_i = 0, _a = executionInfo.unresolved_value_list; _i < _a.length; _i++) {
                                        unresolved_value = _a[_i];
                                        bufferView[unresolved_value.offset] = placeholderContext.resolve(unresolved_value.placeholder);
                                    }
                                    return [2 /*return*/, placeholderContext.resolve(executionInfo)];
                                });
                            }); }))];
                    case 2:
                        // resolve placeholders in execution info
                        _a.executionInfos = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Get input [[webdnn.SymbolicFloat32Array|`SymbolicFloat32Array`]] object
     *
     * @returns array of input [[webdnn.SymbolicFloat32Array|`SymbolicFloat32Array`]]
     * @deprecated Use [[webdnn.DescriptorRunner.inputs|`inputs`]] instead
     */
    DescriptorRunnerWebMetal.prototype.getInputViews = function () {
        if (this.inputs)
            return this.inputs;
        if (!this.descriptor)
            throw new Error('Descriptor is not loaded');
        if (!this.placeholderContext)
            throw new Error('PlaceholderContext is not initialized');
        var descriptor = this.descriptor;
        var placeholderContext = this.placeholderContext;
        this.inputs = descriptor.inputs.map(function (name) {
            var allocation = descriptor.memory_layout.static.allocations[name] || descriptor.memory_layout.dynamic.allocations[name];
            var view = new symbolic_float32array_1.default(null, allocation.offset * symbolic_float32array_1.default.BYTES_PER_ELEMENT, allocation.size, placeholderContext);
            return view;
        });
        return this.inputs;
    };
    /**
     * Get output [[webdnn.SymbolicFloat32Array|`SymbolicFloat32Array`]] object
     *
     * @returns array of output [[webdnn.SymbolicFloat32Array|`SymbolicFloat32Array`]]
     * @deprecated Use [[webdnn.DescriptorRunner.outputs|`outputs`]] instead
     */
    DescriptorRunnerWebMetal.prototype.getOutputViews = function () {
        if (this.outputs)
            return this.outputs;
        if (!this.descriptor)
            throw new Error('Descriptor is not loaded');
        if (!this.placeholderContext)
            throw new Error('PlaceholderContext is not initialized');
        var descriptor = this.descriptor;
        var placeholderContext = this.placeholderContext;
        this.outputs = descriptor.outputs.map(function (name) {
            var allocation = descriptor.memory_layout.static.allocations[name] || descriptor.memory_layout.dynamic.allocations[name];
            var view = new symbolic_float32array_1.default(null, allocation.offset * symbolic_float32array_1.default.BYTES_PER_ELEMENT, allocation.size, placeholderContext);
            return view;
        });
        return this.outputs;
    };
    /**
     * Run descriptor. You must call [[webdnn.DescriptorRunner.getInputViews|`getInputViews`]] and
     * [[webdnn.DescriptorRunner.getOutputViews|`getOutputViews`]] before calling this function.
     */
    DescriptorRunnerWebMetal.prototype.run = function () {
        return __awaiter(this, void 0, void 0, function () {
            var staticBuffer, dynamicBuffer, metaBuffers, records, totalElapsedTime_1, i, exec_info, start, elapsedTime, summary, complete_promise, i, exec_info, is_last;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.executionInfos)
                            throw new Error('ExecutionInfos is not loaded');
                        if (!this.staticBuffer)
                            throw new Error('StaticBuffer is not initialized');
                        if (!this.dynamicBuffer)
                            throw new Error('DynamicBuffer is not initialized');
                        if (!this.metaBuffers)
                            throw new Error('MetaBuffer is not initialized');
                        if (!this.placeholderContext)
                            throw new Error('PlaceholderContext is not initialized');
                        if (!this.placeholderContext.isResolved)
                            throw new Error("Not all placeholders are resolved: " + this.placeholderContext);
                        staticBuffer = this.staticBuffer;
                        dynamicBuffer = this.dynamicBuffer;
                        metaBuffers = this.metaBuffers;
                        if (!webdnn_1.getConfiguration('DEBUG', false)) return [3 /*break*/, 5];
                        records = [];
                        totalElapsedTime_1 = 0;
                        i = 0;
                        _a.label = 1;
                    case 1:
                        if (!(i < this.executionInfos.length)) return [3 /*break*/, 4];
                        exec_info = this.executionInfos[i];
                        start = performance.now();
                        return [4 /*yield*/, this.webmetalHandler.executeSinglePipelineState('descriptor.' + exec_info.entry_func_name, exec_info.threadgroups_per_grid, exec_info.threads_per_thread_group, [staticBuffer, dynamicBuffer, metaBuffers[i]], true)];
                    case 2:
                        _a.sent();
                        elapsedTime = performance.now() - start;
                        records.push({
                            'Kernel': exec_info.entry_func_name,
                            'Elapsed time [ms]': elapsedTime
                        });
                        totalElapsedTime_1 += elapsedTime;
                        _a.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4:
                        summary = Array.from(Object.values(records.reduce(function (summary, record) {
                            if (!(record['Kernel'] in summary)) {
                                summary[record['Kernel']] = {
                                    'Kernel': record['Kernel'],
                                    'Count': 0,
                                    'Elapsed time [ms]': 0,
                                };
                            }
                            summary[record['Kernel']]['Count']++;
                            summary[record['Kernel']]['Elapsed time [ms]'] += record['Elapsed time [ms]'];
                            return summary;
                        }, {})));
                        summary.forEach(function (record) { return record['Ratio [%]'] = (record['Elapsed time [ms]'] / totalElapsedTime_1).toFixed(2); });
                        console.table(records);
                        console.table(summary);
                        return [3 /*break*/, 6];
                    case 5:
                        complete_promise = null;
                        for (i = 0; i < this.executionInfos.length; i++) {
                            exec_info = this.executionInfos[i];
                            is_last = i == this.executionInfos.length - 1;
                            complete_promise = this.webmetalHandler.executeSinglePipelineState('descriptor.' + exec_info.entry_func_name, exec_info.threadgroups_per_grid, exec_info.threads_per_thread_group, [staticBuffer, dynamicBuffer, metaBuffers[i]], is_last);
                        }
                        return [2 /*return*/, complete_promise]; //wait to finish final kernel
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    return DescriptorRunnerWebMetal;
}(descriptor_runner_1.DescriptorRunner));
exports.default = DescriptorRunnerWebMetal;
