"use strict";
/**
 * @module webdnn/math
 */
Object.defineProperty(exports, "__esModule", { value: true });
/** Don't Remove This comment block */
/**
 * Return indices of the top-K largest elements.
 * This implementation is not stable sort.
 *
 * @param {number[]|Float32Array} arr array
 * @param {number} k number of indices
 * @returns {number[]} indices of top-K largest samples
 */
function argmax(arr, k) {
    // Top-k Quicksort
    if (k === void 0) { k = 1; }
    arr = arr.slice();
    var stack = [[0, arr.length]];
    var workspace = [];
    for (var i = 0; i < arr.length; i++)
        workspace[i] = i;
    while (stack.length > 0) {
        var _a = stack.pop(), from = _a[0], to = _a[1], pivot = arr[to - 1], left = from, right = to - 2, tmp = void 0;
        if (from >= to - 1)
            continue;
        while (true) {
            while (arr[left] > pivot && left <= right)
                left++;
            while (arr[right] <= pivot && left <= right)
                right--;
            if (left >= right)
                break;
            tmp = arr[left];
            arr[left] = arr[right];
            arr[right] = tmp;
            tmp = workspace[left];
            workspace[left] = workspace[right];
            workspace[right] = tmp;
        }
        arr[to - 1] = arr[left];
        arr[left] = pivot;
        tmp = workspace[to - 1];
        workspace[to - 1] = workspace[left];
        workspace[left] = tmp;
        // If partial segment contains top-K elements, append it into stack
        stack.push([from, left]); // left (=larger) segment always contains top-K elements
        if (left + 1 < k)
            stack.push([left + 1, to]);
    }
    var result = [];
    for (var i = 0; i < k; i++)
        result.push(workspace[i]);
    return result;
}
exports.argmax = argmax;
/**
 * Return indices of the top-K smallest elements.
 * This implementation is not stable sort.
 *
 * @param {number[]|Float32Array|Int32Array} arr array
 * @param {number} k number of indices
 * @returns {number[]} indices of top-K smallest samples
 */
function argmin(arr, k) {
    // Top-k Quicksort
    if (k === void 0) { k = 1; }
    arr = arr.slice();
    var stack = [[0, arr.length]];
    var workspace = [];
    for (var i = 0; i < arr.length; i++)
        workspace[i] = i;
    while (stack.length > 0) {
        var _a = stack.pop(), from = _a[0], to = _a[1], pivot = arr[to - 1], left = from, right = to - 2, tmp = void 0;
        if (from >= to - 1)
            continue;
        while (true) {
            while (arr[left] < pivot && left <= right)
                left++;
            while (arr[right] >= pivot && left <= right)
                right--;
            if (left >= right)
                break;
            tmp = arr[left];
            arr[left] = arr[right];
            arr[right] = tmp;
            tmp = workspace[left];
            workspace[left] = workspace[right];
            workspace[right] = tmp;
        }
        arr[to - 1] = arr[left];
        arr[left] = pivot;
        tmp = workspace[to - 1];
        workspace[to - 1] = workspace[left];
        workspace[left] = tmp;
        // If partial segment contains top-K elements, append it into stack
        stack.push([from, left]); // left (=larger) segment always contains top-K elements
        if (left + 1 < k)
            stack.push([left + 1, to]);
    }
    var result = [];
    for (var i = 0; i < k; i++)
        result.push(workspace[i]);
    return result;
}
exports.argmin = argmin;
