"use strict";
/**
 * @module webdnn
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * PlaceholderContext manages the placeholders
 * @protected
 */
var PlaceholderContext = /** @class */ (function () {
    function PlaceholderContext(values) {
        this.values = {};
        if (values) {
            this.update(values);
        }
    }
    Object.defineProperty(PlaceholderContext.prototype, "isResolved", {
        get: function () {
            return Object.values(this.values).every(function (value) { return typeof value == 'number'; });
        },
        enumerable: true,
        configurable: true
    });
    PlaceholderContext.prototype.update = function (values) {
        this.values = Object.assign(this.values, values);
    };
    PlaceholderContext.prototype.resolve = function (placeholder) {
        var _this = this;
        // Literal value => return itself.
        if (typeof placeholder !== 'object')
            return placeholder;
        // Placeholder object ( { eval: string } ) => resolve
        if (Object.keys(placeholder).length == 1 && 'eval' in placeholder) {
            if (!this.isResolved)
                throw Error("Not all placeholders are resolved: " + this);
            return eval('(function(placeholders){return ' + placeholder.eval + ';})')(this.values);
        }
        // Array => deep copy
        if (placeholder instanceof Array) {
            return placeholder.map(function (value) { return _this.resolve(value); });
        }
        // Object => deep copy
        return Object.entries(placeholder)
            .reduce(function (result, _a) {
            var key = _a[0], value = _a[1];
            result[key] = _this.resolve(value);
            return result;
        }, {});
    };
    PlaceholderContext.prototype.toString = function () {
        return JSON.stringify(this.values);
    };
    return PlaceholderContext;
}());
exports.default = PlaceholderContext;
