"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @module webdnn
 */
/** Don't Remove This comment block */
var symbolic_typed_array_1 = require("./symbolic_typed_array");
/**
 * Typed array used for input and output variables of [[webdnn.DescriptorRunner| `DescriptorRunner`]].
 * You can use `SymbolicFloat32Array` almost as same as `Float32Array`.
 *
 * To convert `SymbolicFloat32Array` into actual `Float32Array`, use [[webdnn.SymbolicFloat32Array.toActual| `toActual()`]]
 *
 * ```js
 *
 * let result = runner.outputs[0];  //runner.outputs is array of SymbolicFloat32Array
 *
 * // SymbolicFloat32Array does NOT support index access
 * console.log(result[0]);
 * >>> undefined
 *
 * // By conversion, you can access each element by index
 * console.log(result.toActual()[0]);
 * >>> 1.00  // Actual result
 * ```
 */
var SymbolicFloat32Array = /** @class */ (function (_super) {
    __extends(SymbolicFloat32Array, _super);
    function SymbolicFloat32Array() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * The size in bytes of each element in the array.
         */
        _this.BYTES_PER_ELEMENT = 4;
        return _this;
    }
    /**
     * Convert SymbolicTypedArray instance into actual TypedArray instance.
     *
     * @returns actual typed array
     */
    SymbolicFloat32Array.prototype.toActual = function () {
        if (!this.buffer) {
            throw new Error('Internal buffer for this variable is not set. DescriptorRunner.setPlaceholderValue() have to be called before calling this function.');
        }
        return new Float32Array(this.buffer, this.byteOffset, this.length);
    };
    /**
     * Determines whether all the members of an array satisfy the specified test.
     * @param callbackfn A function that accepts up to three arguments. The every method calls
     * the callbackfn function for each element in array1 until the callbackfn returns false,
     * or until the end of the array.
     * @param thisArg An object to which the this keyword can refer in the callbackfn function.
     * If thisArg is omitted, undefined is used as the this value.
     */
    SymbolicFloat32Array.prototype.every = function (callbackfn, thisArg) {
        return this.toActual().every(callbackfn, thisArg);
    };
    /**
     * Returns the elements of an array that meet the condition specified in a callback function.
     * @param callbackfn A function that accepts up to three arguments. The filter method calls
     * the callbackfn function one time for each element in the array.
     * @param thisArg An object to which the this keyword can refer in the callbackfn function.
     * If thisArg is omitted, undefined is used as the this value.
     */
    SymbolicFloat32Array.prototype.filter = function (callbackfn, thisArg) {
        return this.toActual().filter(callbackfn, thisArg);
    };
    /**
     * Returns the value of the first element in the array where predicate is true, and undefined
     * otherwise.
     * @param predicate find calls predicate once for each element of the array, in ascending
     * order, until it finds one where predicate returns true. If such an element is found, find
     * immediately returns that element value. Otherwise, find returns undefined.
     * @param thisArg If provided, it will be used as the this value for each invocation of
     * predicate. If it is not provided, undefined is used instead.
     */
    SymbolicFloat32Array.prototype.find = function (predicate, thisArg) {
        return this.toActual().find(predicate, thisArg);
    };
    /**
     * Returns the index of the first element in the array where predicate is true, and -1
     * otherwise.
     * @param predicate find calls predicate once for each element of the array, in ascending
     * order, until it finds one where predicate returns true. If such an element is found,
     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.
     * @param thisArg If provided, it will be used as the this value for each invocation of
     * predicate. If it is not provided, undefined is used instead.
     */
    SymbolicFloat32Array.prototype.findIndex = function (predicate, thisArg) {
        return this.toActual().findIndex(predicate, thisArg);
    };
    /**
     * Performs the specified action for each element in an array.
     * @param callbackfn  A function that accepts up to three arguments. forEach calls the
     * callbackfn function one time for each element in the array.
     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.
     * If thisArg is omitted, undefined is used as the this value.
     */
    SymbolicFloat32Array.prototype.forEach = function (callbackfn, thisArg) {
        return this.toActual().forEach(callbackfn, thisArg);
    };
    /**
     * Calls a defined callback function on each element of an array, and returns an array that
     * contains the results.
     * @param callbackfn A function that accepts up to three arguments. The map method calls the
     * callbackfn function one time for each element in the array.
     * @param thisArg An object to which the this keyword can refer in the callbackfn function.
     * If thisArg is omitted, undefined is used as the this value.
     */
    SymbolicFloat32Array.prototype.map = function (callbackfn, thisArg) {
        return this.toActual().map(callbackfn, thisArg);
    };
    SymbolicFloat32Array.prototype.reduce = function (callbackfn, initialValue) {
        return this.toActual().reduce(callbackfn, initialValue);
    };
    SymbolicFloat32Array.prototype.reduceRight = function (callbackfn, initialValue) {
        return this.toActual().reduceRight(callbackfn, initialValue);
    };
    /**
     * Reverses the elements in an Array.
     */
    SymbolicFloat32Array.prototype.reverse = function () {
        return this.toActual().reverse();
    };
    /**
     * Returns a section of an array.
     * @param start The beginning of the specified portion of the array.
     * @param end The end of the specified portion of the array.
     */
    SymbolicFloat32Array.prototype.slice = function (start, end) {
        return this.toActual().slice(start, end);
    };
    /**
     * Determines whether the specified callback function returns true for any element of an array.
     * @param callbackfn A function that accepts up to three arguments. The some method calls the
     * callbackfn function for each element in array1 until the callbackfn returns true, or until
     * the end of the array.
     * @param thisArg An object to which the this keyword can refer in the callbackfn function.
     * If thisArg is omitted, undefined is used as the this value.
     */
    SymbolicFloat32Array.prototype.some = function (callbackfn, thisArg) {
        return this.toActual().some(callbackfn, thisArg);
    };
    /**
     * Gets a new Float32Array view of the ArrayBuffer store for this array, referencing the elements
     * at begin, inclusive, up to end, exclusive.
     * @param begin The index of the beginning of the array.
     * @param end The index of the end of the array.
     */
    SymbolicFloat32Array.prototype.subarray = function (begin, end) {
        return this.toActual().subarray(begin, end);
    };
    /** @protected */
    SymbolicFloat32Array.prototype.includes = function (searchElement, fromIndex) {
        return this.toActual().includes(searchElement, fromIndex);
    };
    /**
     * The size in bytes of each element in SymbolicFloat32Array.
     */
    SymbolicFloat32Array.BYTES_PER_ELEMENT = 4;
    return SymbolicFloat32Array;
}(symbolic_typed_array_1.SymbolicTypedArray));
exports.default = SymbolicFloat32Array;
