"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @protected
 */
function flatten(arr) {
    var result = [];
    for (var i = 0; i < arr.length; i++) {
        var v = arr[i];
        if (v instanceof Array) {
            result.splice(result.length, 0, flatten(v));
        }
        else {
            result[result.length] = v;
        }
    }
    return result;
}
/**
 * @protected
 */
var SymbolicTypedArray = /** @class */ (function () {
    /**
     * @protected
     */
    function SymbolicTypedArray(buffer, byteOffset, length, placeholderContext) {
        if (buffer === void 0) { buffer = null; }
        if (byteOffset === void 0) { byteOffset = 0; }
        if (placeholderContext === void 0) { placeholderContext = null; }
        this.placeholderContext = placeholderContext;
        this._byteOffset = byteOffset;
        this._buffer = buffer;
        if (buffer) {
            this._length = length === undefined ? (buffer.byteLength / this.BYTES_PER_ELEMENT) : length;
        }
        else {
            if (length === undefined)
                throw Error('"butter" or "length" must be specified.');
            this._length = length;
        }
        if (this.isDynamic) {
            if (!placeholderContext) {
                throw Error('PlaceholderContext must be required when SymbolicTypedArray is initialized as dynamic buffer view.');
            }
        }
    }
    Object.defineProperty(SymbolicTypedArray.prototype, "buffer", {
        /**
         * The ArrayBuffer instance referenced by the array.
         */
        get: function () {
            if (!this._buffer)
                this._buffer = new ArrayBuffer(this.byteOffset + this.byteLength);
            return this._buffer;
        },
        /**
         * The ArrayBuffer instance referenced by the array.
         */
        set: function (buffer) {
            this._buffer = buffer;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SymbolicTypedArray.prototype, "byteLength", {
        /**
         * The length in bytes of the array.
         */
        get: function () {
            return this.length * this.BYTES_PER_ELEMENT;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SymbolicTypedArray.prototype, "offset", {
        /**
         * The number in this buffer. Actual offset size is `(offset * SIZE_OF_FLOAT)`.
         */
        get: function () {
            return this.byteOffset / this.BYTES_PER_ELEMENT;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SymbolicTypedArray.prototype, "isDynamic", {
        /**
         * @protected
         */
        get: function () {
            return (typeof this._byteOffset !== 'number' || typeof this._length !== 'number');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SymbolicTypedArray.prototype, "length", {
        /**
         * The number of elements in this buffer. Actual byte size is `(length * SIZE_OF_FLOAT)`.
         */
        get: function () {
            if (this.isDynamic) {
                return this.placeholderContext.resolve(this._length);
            }
            else {
                return this._length;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SymbolicTypedArray.prototype, "byteOffset", {
        /**
         * The offset in bytes of the array.
         */
        get: function () {
            if (this.isDynamic) {
                return this.placeholderContext.resolve(this._byteOffset);
            }
            else {
                return this._byteOffset;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns the this object after copying a section of the array identified by start and end
     * to the same array starting at position target
     * @param target If target is negative, it is treated as length+target where length is the
     * length of the array.
     * @param start If start is negative, it is treated as length+start. If end is negative, it
     * is treated as length+end.
     * @param end If not specified, length of the this object is used as its default value.
     */
    SymbolicTypedArray.prototype.copyWithin = function (target, start, end) {
        this.toActual().copyWithin(target, start, end);
        return this;
    };
    /**
     * Returns the this object after filling the section identified by start and end with value
     * @param value value to fill array section with
     * @param start index to start filling the array at. If start is negative, it is treated as
     * length+start where length is the length of the array.
     * @param end index to stop filling the array at. If end is negative, it is treated as
     * length+end.
     */
    SymbolicTypedArray.prototype.fill = function (value, start, end) {
        this.toActual().fill(value, start, end);
        return this;
    };
    /**
     * Returns the index of the first occurrence of a value in an array.
     * @param searchElement The value to locate in the array.
     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the
     *  search starts at index 0.
     */
    SymbolicTypedArray.prototype.indexOf = function (searchElement, fromIndex) {
        return this.toActual().indexOf(searchElement, fromIndex);
    };
    /**
     * Adds all the elements of an array separated by the specified separator string.
     * @param separator A string used to separate one element of an array from the next in the
     * resulting String. If omitted, the array elements are separated with a comma.
     */
    SymbolicTypedArray.prototype.join = function (separator) {
        return this.toActual().join(separator);
    };
    /**
     * Returns the index of the last occurrence of a value in an array.
     * @param searchElement The value to locate in the array.
     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the
     * search starts at index 0.
     */
    SymbolicTypedArray.prototype.lastIndexOf = function (searchElement, fromIndex) {
        return this.toActual().lastIndexOf(searchElement, fromIndex);
    };
    /**
     * Sorts an array.
     * @param compareFn The name of the function used to determine the order of the elements. If
     * omitted, the elements are sorted in ascending, ASCII character order.
     */
    SymbolicTypedArray.prototype.sort = function (compareFn) {
        this.toActual().sort(compareFn);
        return this;
    };
    SymbolicTypedArray.prototype.includes = function (searchElement, fromIndex) {
        return this.toActual().includes(searchElement, fromIndex);
    };
    /**
     * Sets a value or an array of values.
     * @param array A typed or untyped array of values to set.
     * @param offset The index in the current array at which the values are to be written.
     */
    SymbolicTypedArray.prototype.set = function (array, offset) {
        return this.toActual().set(flatten(array), offset);
    };
    /**
     * Converts a number to a string by using the current locale.
     */
    SymbolicTypedArray.prototype.toLocaleString = function () {
        return this.toActual().toLocaleString();
    };
    /**
     * Returns a string representation of an array.
     */
    SymbolicTypedArray.prototype.toString = function () {
        return this.toActual().toString();
    };
    /** @protected */
    SymbolicTypedArray.prototype[Symbol.iterator] = function () {
        return this.toActual()[Symbol.iterator]();
    };
    /**
     * Returns an iterable of key, value pairs for every entry in the array
     */
    SymbolicTypedArray.prototype.entries = function () {
        return this.toActual().entries();
    };
    /**
     * Returns an iterable of keys in the array
     */
    SymbolicTypedArray.prototype.keys = function () {
        return this.toActual().keys();
    };
    /**
     * Returns an iterable of values in the array
     */
    SymbolicTypedArray.prototype.values = function () {
        return this.toActual().values();
    };
    return SymbolicTypedArray;
}());
exports.SymbolicTypedArray = SymbolicTypedArray;
