"use strict";
/**
 * @module webdnn
 */
/** Don't Remove This comment block */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @private
 */
var NOT_SCHEDULED = -1;
/**
 * Schedule function which is called too much frequently.
 *
 * @private
 */
var DispatchScheduler = /** @class */ (function () {
    function DispatchScheduler() {
        this.scheduledCallbackId = NOT_SCHEDULED;
    }
    /**
     * Register scheduled function. If already other function is scheduled, it is canceled and dispatcher will dispatch only
     * function which is registered at last.
     * @param fn scheduled function
     */
    DispatchScheduler.prototype.request = function (fn) {
        var _this = this;
        this.fn = fn;
        if (this.scheduledCallbackId == NOT_SCHEDULED) {
            this.scheduledCallbackId = requestAnimationFrame(function () { return _this.forceDispatch(); });
        }
    };
    /**
     * Dispatch scheduled function just now. If no function is scheduled, dispatcher do nothing.
     */
    DispatchScheduler.prototype.forceDispatch = function () {
        if (this.scheduledCallbackId == NOT_SCHEDULED)
            return;
        this.cancel();
        this.fn();
    };
    /**
     * Cancel scheduled function. If no function is scheduled, dispatcher do nothing.
     */
    DispatchScheduler.prototype.cancel = function () {
        if (this.scheduledCallbackId == NOT_SCHEDULED)
            return;
        cancelAnimationFrame(this.scheduledCallbackId);
        this.scheduledCallbackId = NOT_SCHEDULED;
    };
    return DispatchScheduler;
}());
exports.default = DispatchScheduler;
