"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var descriptor_runner_fallback_1 = require("./descriptor_runner/descriptor_runner_fallback");
var descriptor_runner_webassembly_1 = require("./descriptor_runner/descriptor_runner_webassembly");
var descriptor_runner_webgl_1 = require("./descriptor_runner/descriptor_runner_webgl");
var descriptor_runner_webmetal_1 = require("./descriptor_runner/descriptor_runner_webmetal");
var Image = require("./image");
exports.Image = Image;
var Math = require("./math");
exports.Math = Math;
/**
 * DEBUG flag for developing WebDNN
 * @private
 */
var configurations = {};
/**
 * get configuration
 * @private
 */
function getConfiguration(key, defaultValue) {
    return key in configurations ? configurations[key] : defaultValue;
}
exports.getConfiguration = getConfiguration;
/**
 * set configuration
 * @private
 */
function setConfiguration(key, value) {
    configurations[key] = value;
}
exports.setConfiguration = setConfiguration;
/**
 * Backend constructor map
 * @private
 */
var descriptorRunners = {
    webgpu: descriptor_runner_webmetal_1.default,
    webgl: descriptor_runner_webgl_1.default,
    webassembly: descriptor_runner_webassembly_1.default,
    fallback: descriptor_runner_fallback_1.default
};
/**
 * Check each computing backend is available or not in this browser.
 * The result will be returned as [[BackendAvailability|`BackendAvailability`]] structure.
 *
 * @returns backend availability
 */
function getBackendAvailability() {
    var status = {
        'webgpu': descriptorRunners['webgpu'].checkAvailability(),
        'webgl': descriptorRunners['webgl'].checkAvailability(),
        'webassembly': descriptorRunners['webassembly'].checkAvailability(),
        'fallback': descriptorRunners['fallback'].checkAvailability(),
    };
    var order = ['webgpu', 'webgl', 'webassembly', 'fallback'].filter(function (backend) { return status[backend]; });
    return {
        status: status,
        defaultOrder: order
    };
}
exports.getBackendAvailability = getBackendAvailability;
/**
 * Initialize specified backend
 * @private
 */
function initBackend(backendName, option) {
    return __awaiter(this, void 0, void 0, function () {
        var runner, ex_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(backendName in descriptorRunners))
                        throw new Error("Unknown backend: \"" + backendName + "\"");
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    runner = new descriptorRunners[backendName](option);
                    return [4 /*yield*/, runner.init()];
                case 2:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    ex_1 = _a.sent();
                    console.warn("Failed to initialize " + backendName + " backend: " + ex_1);
                    return [2 /*return*/, null];
                case 4: return [2 /*return*/, runner];
            }
        });
    });
}
/**
 * Initialize descriptor runner. This function performs follow things.
 *
 * 1. Try to initialize computing backend. WebDNN will try to initialize each backend in order of
 *    the result of [[getBackendAvailability|`getBackendAvailability`]].
 *    If you want to modify this order, specify [[InitOption.backendOrder|`initOption.backendOrder`]] option.
 *
 * 2. Load model data based on initialized backend. Generally, DNN binary data is very large and it takes long time to load.
 *    [[InitOption.progressCallback|`initOption.progressCallback`]] option provides the progress status of loading.
 *
 * ### Examples
 *
 * - Basic usage
 *
 *   ```js
 *   let runner = await WebDNN.load('./model');
 *   ```
 *
 * - With `initOption.progressCallback` option
 *
 *   ```js
 *   let runner = await WebDNN.load('./model', {
 *       progressCallback: (loaded, total) => console.log(`${ (loaded/total*100).toFixed(1) }% Loaded`);
 *   });
 *   ```
 *
 * @param directory URL of directory that contains graph descriptor files (e.g. graph_webgpu.json)
 * @param initOption Initialize option
 * @return DescriptorRunner instance, which is the interface to input/output data and run the model.
 */
function load(directory, initOption) {
    if (initOption === void 0) { initOption = {}; }
    return __awaiter(this, void 0, void 0, function () {
        var _a, backendOrder, _b, backendOptions, _c, cacheStrategy, _d, saveCache, progressCallback, weightDirectory, transformUrlDelegate, customTransformUrlDelegate, backendName, options, runner, descriptor, parameters, fetchedDescriptor, cachedDescriptor, _e, e_1, ex_2;
        return __generator(this, function (_f) {
            switch (_f.label) {
                case 0:
                    _a = initOption.backendOrder, backendOrder = _a === void 0 ? null : _a, _b = initOption.backendOptions, backendOptions = _b === void 0 ? {} : _b, _c = initOption.cacheStrategy, cacheStrategy = _c === void 0 ? 'latest' : _c, _d = initOption.saveCache, saveCache = _d === void 0 ? true : _d, progressCallback = initOption.progressCallback, weightDirectory = initOption.weightDirectory, transformUrlDelegate = initOption.transformUrlDelegate;
                    if (!backendOrder)
                        backendOrder = getBackendAvailability().defaultOrder;
                    if (typeof backendOrder === 'string')
                        backendOrder = [backendOrder];
                    backendOrder = backendOrder.slice();
                    if (backendOrder.indexOf('fallback') === -1)
                        backendOrder.concat(['fallback']);
                    customTransformUrlDelegate = function (url) {
                        if (weightDirectory) {
                            if ((/\.bin/).test(url)) {
                                url = url.replace(directory, weightDirectory);
                            }
                        }
                        if (transformUrlDelegate)
                            url = transformUrlDelegate(url);
                        return url;
                    };
                    _f.label = 1;
                case 1:
                    if (!(backendOrder.length > 0)) return [3 /*break*/, 36];
                    backendName = backendOrder.shift();
                    options = __assign({}, backendOptions[backendName]);
                    options.transformUrlDelegate = customTransformUrlDelegate;
                    return [4 /*yield*/, initBackend(backendName, options)];
                case 2:
                    runner = _f.sent();
                    if (!runner)
                        return [3 /*break*/, 1];
                    _f.label = 3;
                case 3:
                    _f.trys.push([3, 34, , 35]);
                    descriptor = void 0;
                    parameters = void 0;
                    fetchedDescriptor = void 0;
                    cachedDescriptor = void 0;
                    _e = cacheStrategy;
                    switch (_e) {
                        case 'latest': return [3 /*break*/, 4];
                        case 'networkOnly': return [3 /*break*/, 13];
                        case 'networkFirst': return [3 /*break*/, 13];
                        case 'cacheOnly': return [3 /*break*/, 20];
                        case 'cacheFirst': return [3 /*break*/, 20];
                    }
                    return [3 /*break*/, 27];
                case 4: return [4 /*yield*/, runner.fetchDescriptor(directory).catch(function () { return null; })];
                case 5:
                    fetchedDescriptor = _f.sent();
                    return [4 /*yield*/, runner.restoreCachedDescriptor(directory)];
                case 6:
                    cachedDescriptor = _f.sent();
                    if (!(cachedDescriptor && fetchedDescriptor && cachedDescriptor.converted_at === fetchedDescriptor.converted_at)) return [3 /*break*/, 8];
                    descriptor = cachedDescriptor;
                    return [4 /*yield*/, runner.restoreCachedParameters(directory, progressCallback)];
                case 7:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 8;
                case 8:
                    if (!fetchedDescriptor) return [3 /*break*/, 10];
                    descriptor = fetchedDescriptor;
                    return [4 /*yield*/, runner.fetchParameters(directory, progressCallback)];
                case 9:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 10;
                case 10:
                    if (!cachedDescriptor) return [3 /*break*/, 12];
                    descriptor = cachedDescriptor;
                    return [4 /*yield*/, runner.restoreCachedParameters(directory, progressCallback)];
                case 11:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 12;
                case 12: throw Error('Network error is occurred and no cache is exist.');
                case 13: return [4 /*yield*/, runner.fetchDescriptor(directory).catch(function () { return null; })];
                case 14:
                    fetchedDescriptor = _f.sent();
                    if (!fetchedDescriptor) return [3 /*break*/, 16];
                    descriptor = fetchedDescriptor;
                    return [4 /*yield*/, runner.fetchParameters(directory, progressCallback)];
                case 15:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 16;
                case 16:
                    if (cacheStrategy === 'networkOnly')
                        throw Error('Network error is occurred in "networkOnly" cache strategy');
                    return [4 /*yield*/, runner.restoreCachedDescriptor(directory)];
                case 17:
                    cachedDescriptor = _f.sent();
                    if (!cachedDescriptor) return [3 /*break*/, 19];
                    descriptor = cachedDescriptor;
                    return [4 /*yield*/, runner.restoreCachedParameters(directory, progressCallback)];
                case 18:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 19;
                case 19: throw Error('Network error is occurred and no cache is exist.');
                case 20: return [4 /*yield*/, runner.restoreCachedDescriptor(directory)];
                case 21:
                    cachedDescriptor = _f.sent();
                    if (!cachedDescriptor) return [3 /*break*/, 23];
                    descriptor = cachedDescriptor;
                    return [4 /*yield*/, runner.restoreCachedParameters(directory, progressCallback)];
                case 22:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 23;
                case 23:
                    if (cacheStrategy === 'cacheOnly')
                        throw Error('No cache is exist in "cacheOnly" cache strategy');
                    return [4 /*yield*/, runner.fetchDescriptor(directory).catch(function () { return null; })];
                case 24:
                    fetchedDescriptor = _f.sent();
                    if (!fetchedDescriptor) return [3 /*break*/, 26];
                    descriptor = fetchedDescriptor;
                    return [4 /*yield*/, runner.fetchParameters(directory, progressCallback)];
                case 25:
                    parameters = _f.sent();
                    if (parameters)
                        return [3 /*break*/, 28];
                    _f.label = 26;
                case 26: throw Error('Network error is occurred and no cache is exist.');
                case 27: throw Error("\"" + cacheStrategy + "\" is not valid cache strategy name: \"latest\", \"networkFirst\", \"networkOnly\", \"cacheFirst\", \"cacheOnly\" is available.");
                case 28:
                    if (!saveCache) return [3 /*break*/, 32];
                    _f.label = 29;
                case 29:
                    _f.trys.push([29, 31, , 32]);
                    return [4 /*yield*/, runner.saveCache(directory, descriptor, parameters)];
                case 30:
                    _f.sent();
                    return [3 /*break*/, 32];
                case 31:
                    e_1 = _f.sent();
                    return [3 /*break*/, 32];
                case 32: return [4 /*yield*/, runner.setDescriptorAndParameters(descriptor, parameters)];
                case 33:
                    _f.sent();
                    return [3 /*break*/, 35];
                case 34:
                    ex_2 = _f.sent();
                    console.warn("Model loading failed for " + backendName + " backend. Trying next backend: " + ex_2.message);
                    return [3 /*break*/, 1];
                case 35: return [2 /*return*/, runner];
                case 36: throw new Error('No backend is available');
            }
        });
    });
}
exports.load = load;
