"use strict";
/**
 * @module webdnn
 */
/** Don't Remove This comment block */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
/// <reference path="./webgl2.d.ts" />
var webdnn_1 = require("./webdnn");
/**
 * @protected
 */
function isWebGL2(gl) {
    return gl.constructor.name === 'WebGL2RenderingContext';
}
exports.isWebGL2 = isWebGL2;
/**
 * @private
 */
var instance;
/**
 * @protected
 */
var WebGLHandler = /** @class */ (function () {
    /**
     * WebGLHandler is singleton class and instantiate directly is forbidden (constructor is hidden).
     *
     * Since the number of GPU contexts may be limited, the handler is used as a singleton
     * and only one context is shared among multiple runners.
     */
    function WebGLHandler() {
        this.gl = checkNull(WebGLHandler.initializeContext());
    }
    WebGLHandler.getInstance = function () {
        if (!instance)
            instance = new WebGLHandler();
        return instance;
    };
    WebGLHandler.prototype.createTexture = function (textureWidth, textureHeight, internalFormat, format) {
        var gl = this.gl;
        var texture = checkNull(gl.createTexture());
        gl.activeTexture(gl.TEXTURE0 + 9); // TODO: texture unit 9 is always available?
        gl.bindTexture(gl.TEXTURE_2D, texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, internalFormat, textureWidth, textureHeight, 0, format, gl.FLOAT, null);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.bindTexture(gl.TEXTURE_2D, null);
        return texture;
    };
    WebGLHandler.prototype.createVertexShader = function (source) {
        return this.createShader(this.gl.VERTEX_SHADER, source);
    };
    WebGLHandler.prototype.createFragmentShader = function (source) {
        return this.createShader(this.gl.FRAGMENT_SHADER, source);
    };
    WebGLHandler.prototype.createShader = function (type, source) {
        var shader = checkNull(this.gl.createShader(type));
        this.gl.shaderSource(shader, source);
        this.gl.compileShader(shader);
        if (!this.gl.getShaderParameter(shader, this.gl.COMPILE_STATUS)) {
            console.error(this.gl.getShaderInfoLog(shader));
            throw Error("Shader Compile failed: " + this.gl.getShaderInfoLog(shader));
        }
        return shader;
    };
    WebGLHandler.prototype.createProgram = function (vertexShader, fragmentShader) {
        var program = checkNull(this.gl.createProgram());
        this.gl.attachShader(program, fragmentShader);
        this.gl.attachShader(program, vertexShader);
        this.gl.linkProgram(program);
        if (!this.gl.getProgramParameter(program, this.gl.LINK_STATUS)) {
            console.error(this.gl.getProgramInfoLog(program));
            throw Error('ShaderProgram Initialization failed.');
        }
        return program;
    };
    WebGLHandler.prototype.createArrayBuffer = function (vertexArray) {
        var buffer = checkNull(this.gl.createBuffer());
        this.gl.bindBuffer(this.gl.ARRAY_BUFFER, buffer);
        this.gl.bufferData(this.gl.ARRAY_BUFFER, vertexArray, this.gl.STATIC_DRAW);
        return buffer;
    };
    WebGLHandler.prototype.createFrameBuffer = function () {
        return checkNull(this.gl.createFramebuffer());
    };
    WebGLHandler.prototype.bindArrayBuffer = function (buffer) {
        this.gl.bindBuffer(this.gl.ARRAY_BUFFER, buffer);
    };
    WebGLHandler.prototype.bindFrameBuffer = function (frameBuffer, width, height) {
        this.gl.bindFramebuffer(this.gl.FRAMEBUFFER, frameBuffer);
        this.gl.viewport(0, 0, width, height);
        this.gl.scissor(0, 0, width, height);
    };
    WebGLHandler.prototype.useProgram = function (program) {
        this.gl.useProgram(program);
    };
    WebGLHandler.prototype.deleteTexture = function (texture) {
        this.gl.deleteTexture(texture);
    };
    WebGLHandler.initializeWebGL2Context = function (canvas) {
        if (canvas === void 0) { canvas = document.createElement('canvas'); }
        var gl;
        gl = (canvas.getContext('webgl2'));
        if (!gl)
            return null;
        if (!gl.getExtension('EXT_color_buffer_float'))
            return null;
        if (webdnn_1.getConfiguration('DEBUG', false) && !gl.getExtension('WEBGL_debug_renderer_info'))
            return null;
        return gl;
    };
    WebGLHandler.initializeWebGL1Context = function (canvas) {
        if (canvas === void 0) { canvas = document.createElement('canvas'); }
        var gl = (canvas.getContext('webgl') || canvas.getContext('experimental-webgl'));
        if (!gl)
            return null;
        if (!gl.getExtension('OES_texture_float'))
            return null;
        if (WebGLHandler.IS_SAFARI) {
            //TODO(Kiikurage)
            // Safari supports WebGL with OES_TEXTURE_FLOAT extension. However,
            // currently when WebGLRenderingContext#readPixels is called, an error is thrown.
            return null;
        }
        if (webdnn_1.getConfiguration('DEBUG', false) && !gl.getExtension('WEBGL_debug_renderer_info'))
            return null;
        return gl;
    };
    WebGLHandler.initializeContext = function () {
        var canvas = document.createElement('canvas');
        var gl;
        gl = WebGLHandler.initializeWebGL2Context(canvas);
        if (gl) {
            if (webdnn_1.getConfiguration('DEBUG', false))
                console.info('WebGL2 is enabled');
        }
        else {
            gl = WebGLHandler.initializeWebGL1Context(canvas);
            if (gl) {
                if (webdnn_1.getConfiguration('DEBUG', false))
                    console.info('WebGL2 is disabled');
            }
            else {
                return null;
            }
        }
        gl.disable(gl.DEPTH_TEST);
        gl.disable(gl.STENCIL_TEST);
        gl.disable(gl.BLEND);
        gl.disable(gl.DITHER);
        gl.disable(gl.POLYGON_OFFSET_FILL);
        gl.disable(gl.SAMPLE_COVERAGE);
        gl.enable(gl.SCISSOR_TEST);
        gl.enable(gl.CULL_FACE);
        gl.cullFace(gl.BACK);
        return gl;
    };
    /**
     * Check whether WebGL is supported or not
     * @protected
     */
    WebGLHandler.checkAvailability = function () {
        if (availability === null) {
            var gl = WebGLHandler.initializeContext();
            if (!gl) {
                availability = false;
            }
            else if (webdnn_1.getConfiguration('MAX_TEXTURE_SIZE', gl.getParameter(gl.MAX_TEXTURE_SIZE)) < 4096) {
                availability = false;
            }
            else {
                availability = true;
            }
        }
        return availability;
    };
    WebGLHandler.prototype.waitForComplete = function () {
        return __awaiter(this, void 0, void 0, function () {
            var gl, sync, status_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        gl = this.gl;
                        if (!isWebGL2(gl)) return [3 /*break*/, 4];
                        sync = gl.fenceSync(gl.SYNC_GPU_COMMANDS_COMPLETE, 0);
                        status_1 = gl.clientWaitSync(sync, 0, 0);
                        _a.label = 1;
                    case 1:
                        if (!(status_1 !== gl.CONDITION_SATISFIED && status_1 !== gl.ALREADY_SIGNALED)) return [3 /*break*/, 3];
                        return [4 /*yield*/, new Promise(function (r) { return setTimeout(r, 1); })];
                    case 2:
                        _a.sent();
                        status_1 = gl.clientWaitSync(sync, 0, 0);
                        return [3 /*break*/, 1];
                    case 3:
                        gl.deleteSync(sync);
                        return [3 /*break*/, 5];
                    case 4:
                        gl.finish();
                        _a.label = 5;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    Object.defineProperty(WebGLHandler.prototype, "MAX_TEXTURE_SIZE", {
        get: function () {
            var MAX_TEXTURE_SIZE = webdnn_1.getConfiguration('MAX_TEXTURE_SIZE', this.gl.getParameter(this.gl.MAX_TEXTURE_SIZE));
            // FIXME: In most case, MAX_TEXTURE_SIZE=4096 is the fastest (Why?).
            if (MAX_TEXTURE_SIZE >= 16384) {
                return 4096;
            }
            else if (MAX_TEXTURE_SIZE >= 8192) {
                return 4096;
            }
            else if (MAX_TEXTURE_SIZE >= 4096) {
                return 4096;
            }
            else {
                throw new Error("MAX_TEXTURE_SIZE is too small: " + MAX_TEXTURE_SIZE);
            }
        },
        enumerable: true,
        configurable: true
    });
    WebGLHandler.IS_SAFARI = navigator.userAgent.toLowerCase().indexOf('safari') !== -1 && navigator.userAgent.toLowerCase().indexOf('chrome') === -1;
    return WebGLHandler;
}());
exports.default = WebGLHandler;
/**
 * @private
 */
var availability = null;
/**
 * @private
 */
function checkNull(obj) {
    if (obj === null)
        throw Error('Null is detected');
    return obj;
}
