"use strict";
///<reference path="./webmetal.ts" />
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @module webdnn
 */
/** Don't Remove This comment block */
var buffer_webmetal_1 = require("./buffer/buffer_webmetal");
/**
 * @private
 */
var instance;
/**
 * @protected
 */
var WebMetalHandler = /** @class */ (function () {
    /**
     * WebMetalHandler is singleton class and instantiate directly is forbidden (constructor is hidden).
     *
     * Since the number of GPU contexts may be limited, the handler is used as a singleton
     * and only one context is shared among multiple runners.
     */
    function WebMetalHandler() {
        this.pipelineStates = new Map();
        if (!exports.IS_WEBMETAL_SUPPORTED)
            throw new Error('This browser does not support WebMetal');
        var context;
        try {
            if (exports.IS_APPLE_WEBGPU_SUPPORTED) {
                // for compatibility, get context by 'webgpu'
                context = document.createElement('canvas').getContext('webgpu');
            }
            else {
                context = document.createElement('canvas').getContext('webmetal');
            }
        }
        catch (err) {
            throw new Error("During initializing WebMetalRenderingContext, unexpected error is occurred: " + err.message);
        }
        if (!context)
            throw new Error('WebMetalRenderingContext initialization failed');
        this.context = context;
        this.commandQueue = context.createCommandQueue();
        this.loadKernel('kernel void sync(){}', 'basic');
    }
    WebMetalHandler.getInstance = function () {
        if (!instance)
            instance = new WebMetalHandler();
        return instance;
    };
    WebMetalHandler.prototype.createBuffer = function (arrayBuffer) {
        return this.context.createBuffer(arrayBuffer);
    };
    WebMetalHandler.prototype.loadKernel = function (librarySource, namespace) {
        if (namespace === void 0) { namespace = ''; }
        var library = this.context.createLibrary(librarySource);
        for (var _i = 0, _a = library.functionNames; _i < _a.length; _i++) {
            var name_1 = _a[_i];
            var kernelFunction = library.functionWithName(name_1);
            var pipelineStates = this.context.createComputePipelineState(kernelFunction);
            this.pipelineStates.set(namespace + '.' + name_1, pipelineStates);
        }
    };
    WebMetalHandler.prototype.createCommandBuffer = function () {
        return this.commandQueue.createCommandBuffer();
    };
    WebMetalHandler.prototype.getPipelineStateByName = function (name) {
        var state = this.pipelineStates.get(name);
        if (!state) {
            throw TypeError("Kernel function \"" + name + "\" is not loaded.");
        }
        return state;
    };
    WebMetalHandler.prototype.executeSinglePipelineState = function (name, threadgroupsPerGrid, threadsPerThreadgroup, buffers, getCompletedPromise) {
        var commandBuffer = this.commandBuffer || (this.commandBuffer = this.createCommandBuffer());
        var commandEncoder = commandBuffer.createComputeCommandEncoder();
        commandEncoder.setComputePipelineState(this.getPipelineStateByName(name));
        for (var i = 0; i < buffers.length; i++) {
            var buffer = buffers[i];
            var wgbuf = void 0;
            if (buffer instanceof buffer_webmetal_1.default) {
                wgbuf = buffer.buffer;
            }
            else {
                // cannot perform (buffer instanceof WebMetalBuffer) currently
                wgbuf = buffer;
            }
            commandEncoder.setBuffer(wgbuf, 0, i);
        }
        commandEncoder.dispatch(threadgroupsPerGrid, threadsPerThreadgroup);
        commandEncoder.endEncoding();
        var promise = null;
        if (getCompletedPromise) {
            promise = commandBuffer.completed;
        }
        this.commandBuffer = null;
        commandBuffer.commit();
        return promise;
    };
    WebMetalHandler.prototype.sync = function () {
        return __awaiter(this, void 0, void 0, function () {
            var commandBuffer, commandEncoder, promise;
            return __generator(this, function (_a) {
                commandBuffer = this.createCommandBuffer();
                commandEncoder = commandBuffer.createComputeCommandEncoder();
                commandEncoder.setComputePipelineState(this.getPipelineStateByName('basic.sync'));
                commandEncoder.dispatch({
                    width: 1,
                    height: 1,
                    depth: 1
                }, {
                    width: 1,
                    height: 1,
                    depth: 1
                });
                commandEncoder.endEncoding();
                promise = commandBuffer.completed;
                commandBuffer.commit();
                return [2 /*return*/, promise];
            });
        });
    };
    return WebMetalHandler;
}());
exports.default = WebMetalHandler;
/**
 * Flag whether WebGPU on Safari is supported or not
 * Its name was changed to WebMetal in 2019 (macOS 10.14.4 / iOS 12.2).
 * This is used for backward compatibility.
 * @protected
 */
exports.IS_APPLE_WEBGPU_SUPPORTED = 'WebGPURenderingContext' in window && 'WebGPUComputeCommandEncoder' in window;
/**
 * Flag whether WebMetal is supported or not
 * @protected
 */
exports.IS_WEBMETAL_SUPPORTED = ('WebMetalRenderingContext' in window && 'WebMetalComputeCommandEncoder' in window) || exports.IS_APPLE_WEBGPU_SUPPORTED;
